Module query

Module query 

This module provides the main query engine implementation, as well as the parser for the query language and the intermediary AST representations of queries.

Re-exports

pub use ast::*;
pub use dfa::*;
pub use parser::*;

Modules

ast
Query AST and Builder
dfa
DFA-based Query Engine
parser
Query Parser

Structs

JSONPointer
A JSON pointer that points to a value in a JSON document.
QueryNFA
Represents a Non-Deterministic Finite Automaton (NFA) for JSON queries. Importantly, the alphabet depends on the query.

Enums

PathType
Represents the type of path being explored in the query.

Traits

QueryEngine
Interface for query engine implementations.

Functions

compute_first_set
Recursively computes the set of letters which occur as the first letter of a word in L(e’).
compute_follows_set
Recursively computes the factors set of letter bigrams that can occur in a word in L(e’).
compute_last_set
Recursively computes the set of letters which occur as the last letter of a word in L(e’).
contains_empty_word
Recursively determines whether the empty word is a member of L(e’).