This module provides the main query engine implementation, as well as the parser for the query language and the intermediary AST representations of queries.
Re-exports
Modules
Structs
- JSON
Pointer - A JSON pointer that points to a value in a JSON document.
- QueryNFA
- Represents a Non-Deterministic Finite Automaton (NFA) for JSON queries. Importantly, the alphabet depends on the query.
Enums
- Path
Type - Represents the type of path being explored in the query.
Traits
- Query
Engine - Interface for query engine implementations.
Functions
- compute_
first_ set - Recursively computes the set of letters which occur as the first letter of a word in L(e’).
- compute_
follows_ set - Recursively computes the factors set of letter bigrams that can occur in a word in L(e’).
- compute_
last_ set - Recursively computes the set of letters which occur as the last letter of a word in L(e’).
- contains_
empty_ word - Recursively determines whether the empty word is a member of L(e’).